"use strict";
/**
 * @fileoverview Prefer pascal case
 * @author Yann Braga
 */
const ast_utils_1 = require("@typescript-eslint/experimental-utils/dist/ast-utils");
const ast_1 = require("../utils/ast");
const constants_1 = require("../utils/constants");
const create_storybook_rule_1 = require("../utils/create-storybook-rule");
module.exports = (0, create_storybook_rule_1.createStorybookRule)({
    name: 'prefer-pascal-case',
    defaultOptions: [],
    meta: {
        type: 'suggestion',
        fixable: 'code',
        docs: {
            description: 'Stories should use PascalCase',
            categories: [constants_1.CategoryId.RECOMMENDED],
            recommended: 'warn',
        },
        messages: {
            convertToPascalCase: 'Use pascal case',
            usePascalCase: 'The story should use PascalCase notation: {{name}}',
        },
        schema: [],
    },
    create(context) {
        // variables should be defined here
        //----------------------------------------------------------------------
        // Helpers
        //----------------------------------------------------------------------
        const isPascalCase = (str) => /^[A-Z]+([a-z0-9]?)+/.test(str);
        const toPascalCase = (str) => {
            return (str
                //@ts-ignore
                .replace(new RegExp(/[-_]+/, 'g'), ' ')
                //@ts-ignore
                .replace(new RegExp(/[^\w\s]/, 'g'), '')
                .replace(
            //@ts-ignore
            new RegExp(/\s+(.)(\w+)/, 'g'), ($1, $2, $3) => `${$2.toUpperCase() + $3.toLowerCase()}`)
                //@ts-ignore
                .replace(new RegExp(/\s/, 'g'), '')
                .replace(new RegExp(/\w/), (s) => s.toUpperCase()));
        };
        //----------------------------------------------------------------------
        // Public
        //----------------------------------------------------------------------
        return {
            ExportNamedDeclaration: function (node) {
                // if there are specifiers, node.declaration should be null
                if (!node.declaration)
                    return;
                const decl = node.declaration;
                if ((0, ast_1.isVariableDeclaration)(decl)) {
                    const { id } = decl.declarations[0];
                    if ((0, ast_1.isIdentifier)(id)) {
                        const { name } = id;
                        if (!isPascalCase(name)) {
                            context.report({
                                node: id,
                                messageId: 'usePascalCase',
                                data: {
                                    name,
                                },
                                suggest: [
                                    {
                                        messageId: 'convertToPascalCase',
                                        *fix(fixer) {
                                            const fullText = context.getSourceCode().text;
                                            const fullName = fullText.slice(id.range[0], id.range[1]);
                                            const suffix = fullName.substring(name.length);
                                            const pascal = toPascalCase(name);
                                            yield fixer.replaceTextRange(id.range, pascal + suffix);
                                            const variable = (0, ast_utils_1.findVariable)(context.getScope(), name);
                                            for (let i = 0; i < variable.references.length; i++) {
                                                const ref = variable.references[i];
                                                if (!ref.init) {
                                                    yield fixer.replaceTextRange(ref.identifier.range, pascal);
                                                }
                                            }
                                        },
                                    },
                                ],
                            });
                        }
                    }
                }
            },
        };
    },
});
